@@ -101,6 +101,9 @@
 //!   - Dependencies are full of unsafe
 //!
 //! # Changelog
+//! - v0.2.0
+//!   - `Executor::new` and `Executor::with_name` to return `Result`.
+//!   - Upgrade to `safina-threadpool` v0.2.0.
 //! - v0.1.7 - `block_on` functions to take futures that are not `Send`.
 //! - v0.1.6 - Fix deadlock in `block_on` and `block_on_unpin` when task is awakened a second time.
 //! - v0.1.5 - Support stable Rust!  Needs 1.51+.
@@ -149,7 +152,7 @@ use core::future::Future;
 use core::pin::Pin;
 use core::task::Poll;
 use safina_sync::Promise;
-use safina_threadpool::ThreadPool;
+use safina_threadpool::{NewThreadPoolError, ThreadPool};
 use std::sync::mpsc::SyncSender;
 use std::sync::{Arc, Mutex, Weak};
 
@@ -208,9 +211,12 @@ pub struct Executor {
 
 impl Executor {
     /// Creates a new executor with 4 async threads and 4 blocking threads.
+    ///
+    /// # Panics
+    /// Panics when it fails to start the threads.
     #[must_use]
     pub fn default() -> Arc<Self> {
-        Self::new(4, 4)
+        Self::new(4, 4).unwrap()
     }
 
     /// Creates a new executor.
@@ -222,8 +228,13 @@ impl Executor {
     /// blocking jobs like connecting TCP sockets and reading files.
     ///
     /// You probably want to use [`default`](#method.default) instead of this.
-    #[must_use]
-    pub fn new(num_async_threads: usize, num_blocking_threads: usize) -> Arc<Self> {
+    ///
+    /// # Errors
+    /// Returns an error when a parameter is invalid or it fails to start threads.
+    pub fn new(
+        num_async_threads: usize,
+        num_blocking_threads: usize,
+    ) -> Result<Arc<Self>, NewThreadPoolError> {
         Self::with_name("async", num_async_threads, "blocking", num_blocking_threads)
     }
 
@@ -232,17 +243,19 @@ impl Executor {
     /// For example, `with_name("api_async", 4, "api_blocking", 100)`
     /// creates 4 threads named `"api_async0"` through `"api_async3"`
     /// and 100 threads named `"api_blocking0"` through `"api_blocking99"`.
-    #[must_use]
+    ///
+    /// # Errors
+    /// Returns an error when a parameter is invalid or it fails to start threads.
     pub fn with_name(
         async_threads_name: &'static str,
         num_async_threads: usize,
         blocking_threads_name: &'static str,
         num_blocking_threads: usize,
-    ) -> Arc<Self> {
-        Arc::new(Self {
-            async_pool: ThreadPool::new(async_threads_name, num_async_threads),
-            blocking_pool: ThreadPool::new(blocking_threads_name, num_blocking_threads),
-        })
+    ) -> Result<Arc<Self>, NewThreadPoolError> {
+        Ok(Arc::new(Self {
+            async_pool: ThreadPool::new(async_threads_name, num_async_threads)?,
+            blocking_pool: ThreadPool::new(blocking_threads_name, num_blocking_threads)?,
+        }))
     }
 
     /// Schedules `f` to run on any available thread in the blocking thread pool.
@@ -293,7 +306,7 @@ impl Executor {
     /// # }
     /// ```
     pub fn spawn(self: &Arc<Self>, fut: impl (Future<Output = ()>) + Send + 'static) {
-        self.spawn_unpin(Box::pin(fut))
+        self.spawn_unpin(Box::pin(fut));
     }
 
     /// Adds a task that will execute `fut`.
@@ -464,7 +477,7 @@ impl std::task::Wake for TaskWaker {
 /// # }
 /// ```
 pub fn spawn(fut: impl (Future<Output = ()>) + Send + 'static) {
-    spawn_unpin(Box::pin(fut))
+    spawn_unpin(Box::pin(fut));
 }
 
 /// Creates a new task to execute `fut` and schedules it for immediate execution.
@@ -489,7 +502,7 @@ pub fn spawn_unpin(fut: impl (Future<Output = ()>) + Send + Unpin + 'static) {
             .async_pool
             .schedule(move || poll_task(task, executor_weak));
     } else {
-        panic!("called from outside a task")
+        panic!("called from outside a task");
     }
 }
 
